;________________________________________________________________________
;
;                            L A D D E R . A S M
;________________________________________________________________________
;
; Sending packet audio tones with a resistor ladder routines
; by Byon Garrabrant  N6BG  byon@mail.com  Jul 98
;________________________________________________________________________
;
	cblock				; variables
		FLAGS			; generic flags / bits
		sine_param
		intWsave		; interrupt flags
		intSTATUSsave
		intPCLATHsave
		freq_add		; packet sending flags
		freq_phase
		new_freq		; next freq to send
		bit_timer		; number of ints to continue tone
	endc
;________________________________________________________________________

#define	SENT	FLAGS, 1
#define	BANK1	H'7F'
;________________________________________________________________________

	ORG	H'0004'

Timer_Interrupt
	movwf	intWsave		; save W
	swapf	STATUS, W		; save STATUS
	movwf	intSTATUSsave
	movfw	PCLATH			; save PCLATH
	movwf	intPCLATHsave


	bcf	STATUS, RP0		; select bank 0 (working with TMR0)
	movlw	D'65'-2			; bump timer forward for
	subwf	TMR0, F			; update TMR0 for next int

	movfw	freq_add		; move the phase forward by add;
	addwf	freq_phase, F

	rrf	freq_phase, W		; get half the param

	movwf	sine_param		; save it
	andlw	H'3F'			; convert 0->127 to 0->63
	btfsc	sine_param, 5		; if >= 32
	 sublw	D'64'			; subtract from 64 (0->31->32->1)
	call	Get_Quarter_Sine	; lookup quarter sine table
	btfsc	sine_param, 6		; if converted above
	 sublw	D'16'			; invert answer

	movwf	sine_param		; save the result
	movfw	PORTB			; read the port
	andlw	H'F0'			; mask off the ladder
	iorwf	sine_param, W		; add in the data
	movwf	PORTB			; and store it

	decfsz	bit_timer, F		; check bit timer
	 goto	timer_end		; in not over, end

	movfw	new_freq		; start the new freq
	movwf	freq_add
	bsf	SENT			; ready for the next bit

	movlw	D'32'			; how long to run this tone
	movwf	bit_timer

timer_end
	bcf	INTCON, T0IF		; enable TMR0 int
	movfw	intPCLATHsave		; restore PCLATH
	movwf	PCLATH			
	swapf	intSTATUSsave, W	; restore STATUS
	movwf	STATUS
	swapf	intWsave, F		; restore W
	swapf	intWsave, W
	retfie				; return from interrupt

;________________________________________________________________________

Get_Quarter_Sine	; w enters 0 to 32, w exits 8 to 15
	addwf	PCL, F
	dt	D'8', D'8', D'9', D'9', D'9', D'10',D'10',D'10'
	dt	D'11',D'11',D'11',D'12',D'12',D'12',D'12',D'13'
	dt	D'13',D'13',D'13',D'14',D'14',D'14',D'14',D'14'
	dt	D'14',D'15',D'15',D'15',D'15',D'15',D'15',D'15'
	dt	D'15'
;________________________________________________________________________

Tone_Init
	clrf	INTCON
	bsf	STATUS, RP0		; Select Bank 1
	movlw	H'F0'
	andwf	PORTB, F
;	bcf	STATUS, RP0		; Select Bank 0
;	return

Tone_Disable
	bcf	INTCON, T0IE		; disableTMR0int
	bsf	STATUS, RP0		; Select Bank 1
	movlw	b'11010111'		; divide TMR0 by 256
	movwf	OPTION_REG & BANK1	; enable TMR0
	bcf	STATUS, RP0		; Select Bank 0
	bcf	INTCON, GIE		; disable ints
	return

Tone_Enable
	movlw	D'8'			; set initial frequency
	movwf	new_freq		
	bsf	STATUS, RP0		; Select Bank 1
	movlw	b'11011000'
	movwf	OPTION_REG & BANK1	; enable TMR0
	bcf	STATUS, RP0		; Select Bank 0
	bsf	INTCON, T0IE		; enableTMR0int
	bsf	INTCON, GIE		; enable ints
	return

Tone_Toggle_If_No_Carry
	btfss	SENT			; wait for last bit to be sent
	 goto	$-1

	movfw	new_freq		
	skpc				; if sending a 0
	 sublw	D'8' + D'15'		;  change frequency
	movwf	new_freq

	bcf 	SENT			; next bit is ready
	return
;________________________________________________________________________

